module Console
where

import Game
import Monad
--import Data.Array hiding (bounds, accumArray)
--import Data.Array.IArray
import Data.Array.Unboxed
import Data.Set
import Data.FiniteMap

printSummary :: Game -> Status -> IO ()
printSummary game status = do
    putStrLn (replicate (1 + sizeXplus1) '=')
    mapM_ (\y -> do
	putChar '|'
	mapM_ (\x -> do
	    when ((x,y) `elemFM` robotIndexStatus  ) $ putStr "\27[7m"
	    when ((x,y) `elemFM` packageGroundIndexStatus) $ putStr "\27[1m"
	    putStr ( (case unpackTerrain (board!(x,y)) of
		       { Plain->'.'; Water->'~'; Wall->'#'; Home->'@' })
		   : "\27[m" ))
	    [1..sizeXplus1-1]
	putStrLn "|")
	[sizeYplus1-1,sizeYplus1-2..1]
    putStrLn (replicate (1 + sizeXplus1) '=')
    mapM_ (\(loc, rs) ->
	putStrLn (show loc ++ ":" ++ concatMap summarizeRobot rs))
	(fmToList robotIndexStatus)
    mapM_ (putStrLn . summarizePackage) (fmToList (packageMap status))
    putStrLn ("Capacity: " ++ show (capacity game) ++ " lbs. " ++
	      "Currency: $" ++ show (myCurrency status) ++ ".")
    return ()

  where
    board			    = currentBoard game
    ((0,0),(sizeXplus1,sizeYplus1)) = bounds board
    robotIndexStatus		    = robotIndex status
    packageGroundIndexStatus	    = packageGroundIndex status
    packageTransitIndexStatus	    = packageTransitIndex status

    summarizeRobot (robotID,_)
      = " " ++
        (if myself then "\27[7m " else "") ++
        show robotID ++
        maybe "" show (lookupFM packageTransitIndexStatus robotID) ++
	(if myself then " \27[m" else "")
      where myself = robotID == myRobotID game

    summarizePackage (id, package)
      = show id ++ ": " ++
	(case packageWeight package of
	    Just wt -> show wt ++ " lbs "
	    Nothing -> "") ++
	(case packageSource package of
	    Just src -> "from " ++ show src ++ " "
	    Nothing -> "") ++
	(case packageTarget package of
	    Left neg | isEmptySet neg -> ""
	    Left neg -> "not-to" ++ concatMap ((' ':).show) (setToList neg) ++ " "
	    Right pos -> "to " ++ show pos ++ " ") ++
        (case packageLocation package of
	    Transit robotID -> "carried by " ++ show robotID
	    Ground loc (Just robotID) -> "dropped at " ++ show loc ++ " by " ++ show robotID
	    Ground loc Nothing -> "at " ++ show loc)
