module Statistics
where

import Game
import Random
import Data.FiniteMap
import Data.Array.IArray
import System.IO.Unsafe (unsafeInterleaveIO)

compilePackageWeight :: Status -> [Weight]
compilePackageWeight Status{ packageMap = packageMap }
  = [ wt | Package{ packageWeight = Just wt } <- eltsFM packageMap ]

compilePackageDistance :: Metric -> Status -> [Currency]
compilePackageDistance metric Status{ packageMap = packageMap }
  = [ metric src tgt
    | Package{ packageSource = Just src, packageTarget = Right tgt }
	<- eltsFM packageMap ]

compileHomePackages :: Game -> Status -> [Int]
compileHomePackages Game{ currentBoard = board }
                 Status{ packageMap = packageMap }
  = map snd
  $ filter (\(src,_) -> unpackTerrain (board!src) == Home)
  $ fmToList
  $ addListToFM_C (+) emptyFM
  $ [ (src,1) | Package{ packageSource = Just src } <- eltsFM packageMap ]

imagineElt :: a -> [a] -> IO a
imagineElt fallback [] = return fallback
imagineElt _ samples = {-unsafeInterleaveIO-} (getStdRandom f)
    where n   = length samples
	  f g = case randomR (0,n-1) g of (i,g') -> (samples!!i,g')
