module Utility
where

import Data.Set
import Data.FiniteMap
import Data.Array.Unboxed

infixr 0 `deepSeq`

prepend :: [a] -> [a] -> [a]
prepend = flip (++)

instance (Show elt) => Show (Set elt) where
    showsPrec p set = showString "Set " . showsPrec p (setToList set)

instance (Show key, Show elt) => Show (FiniteMap key elt) where
    showsPrec p fm = showString "FiniteMap " . showsPrec p (fmToList fm)

class DeepSeq a where
    deepSeq :: a -> b -> b
    deepSeq = seq

instance DeepSeq () where

instance (DeepSeq a) => DeepSeq [a] where
  deepSeq [] y = y
  deepSeq (x:xs) y = deepSeq x $ deepSeq xs y

instance (DeepSeq a,DeepSeq b) => DeepSeq (a,b) where
  deepSeq (a,b) y = deepSeq a $ deepSeq b y
instance (DeepSeq a,DeepSeq b,DeepSeq c) => DeepSeq (a,b,c) where
  deepSeq (a,b,c) y = deepSeq a $ deepSeq b $ deepSeq c y
instance (DeepSeq a,DeepSeq b,DeepSeq c,DeepSeq d) => DeepSeq (a,b,c,d) where
  deepSeq (a,b,c,d) y = deepSeq a $ deepSeq b $ deepSeq c $ deepSeq d y
instance (DeepSeq a,DeepSeq b,DeepSeq c,DeepSeq d,DeepSeq e) => DeepSeq (a,b,c,d,e) where
  deepSeq (a,b,c,d,e) y = deepSeq a $ deepSeq b $ deepSeq c $ deepSeq d $ deepSeq e y
instance (DeepSeq a,DeepSeq b,DeepSeq c,DeepSeq d,DeepSeq e,DeepSeq f) => DeepSeq (a,b,c,d,e,f) where
  deepSeq (a,b,c,d,e,f) y = deepSeq a $ deepSeq b $ deepSeq c $ deepSeq d $ deepSeq e $ deepSeq f y
instance (DeepSeq a,DeepSeq b,DeepSeq c,DeepSeq d,DeepSeq e,DeepSeq f,DeepSeq g) => DeepSeq (a,b,c,d,e,f,g) where
  deepSeq (a,b,c,d,e,f,g) y = deepSeq a $ deepSeq b $ deepSeq c $ deepSeq d $ deepSeq e $ deepSeq f $ deepSeq g y

instance DeepSeq Bool where
instance DeepSeq Char where

instance (DeepSeq a) => DeepSeq (Maybe a) where
  deepSeq Nothing y = y
  deepSeq (Just x) y = deepSeq x y

instance (DeepSeq a, DeepSeq b) => DeepSeq (Either a b) where
  deepSeq (Left a) y = deepSeq a y
  deepSeq (Right b) y = deepSeq b y

instance DeepSeq Ordering where

instance DeepSeq Integer where
instance DeepSeq Int where
instance DeepSeq Float where
instance DeepSeq Double where

instance (DeepSeq elt) => DeepSeq (Set elt) where
  deepSeq = deepSeq . setToList

instance (DeepSeq key, DeepSeq elt) => DeepSeq (FiniteMap key elt) where
  deepSeq = deepSeq . fmToList

instance DeepSeq (UArray i e) where

map2 :: (a -> a -> b) -> [a] -> [b]
map2 f [_] = []
map2 f (x0:x1:xs) = f x0 x1:map2 f xs

merge :: (Ord a) => [a] -> [a] -> [a]
merge [] l = l
merge l [] = l
merge xs@(x0:xs') ys@(y0:ys')
   | x0 == y0 = x0:merge xs' ys'
   | x0 < y0  = x0:merge xs' ys
   | x0 > y0  = y0:merge xs ys'

neighbors :: (Ord a) => [a] -> a -> [a]
neighbors xs x = aux [] xs x
  where aux l [] x = l
	aux l (x0:xs) x | x == x0 = [x]
			| x < x0  = (x0:l)
			| otherwise = aux [x] xs x
